require 'json'
require 'digest'
require 'net/http'

module Sms
  class Taobao

    attr_accessor :app_secret, :app_key,
                  :sms_free_sign_name, :called_show_verify_nums, :called_show_notify_nums,
                  :template_code, :param,
                  :custom_params, :request_params

    def initialize(options = {})
      options.each{|k, v|
        next unless respond_to?(k)
        self.send("#{k.to_s}=", v)
      }
    end

    def single_send(number)
      sms_num_send(number.split)
    end

    def batch_send(numbers)
      sms_num_send(numbers)
    end

    def public_params
      {
        app_key: self.app_key,
        timestamp: Time.now.strftime("%Y-%m-%d %H:%M:%S"),
        format: 'json',
        v: 2.0,
        sign_method: 'md5',
      }
    end

    def extend_params
      params = public_params.merge(self.custom_params)
      self.request_params = params.merge(sign: generate_sign(params))
    end

    def generate_sign(opts = {})
      str = opts.to_a.map{|m| m.map(&:to_s).join }.sort.join
      Digest::MD5.hexdigest("#{app_secret.to_s}#{str}#{app_secret.to_s}").upcase
    end

    def sms_num_send(numbers)
      self.custom_params = {
        method: 'alibaba.aliqin.fc.sms.num.send',
        rec_num: numbers.join(','),
        sms_param: param.to_json,
        sms_template_code: template_code,
        sms_type: 'normal',
        sms_free_sign_name: self.sms_free_sign_name
      }

      post
    end

    def tts_num_singlecall(numbers, type = 'notify')
      called_show_nums = send("called_show_#{type}_nums")
      size = called_show_nums.size

      self.custom_params = {
        method: 'alibaba.aliqin.fc.tts.num.singlecall',
        called_num: numbers.join(','),
        called_show_num: called_show_nums[rand(1..size)],
        tts_param: param.to_json,
        tts_code: template_code
      }

      post
    end

    def post
      res = Net::HTTP.post_form(URI.parse('http://gw.api.taobao.com/router/rest'), extend_params)
      puts "#{request_params[:method]} request params: #{request_params.inspect}"

      data = res.body
      data = JSON.parse(data) if data.is_a? String

      puts "#{request_params[:method]} request result: #{data.inspect}"
      result = HashWithIndifferentAccess.new data

      keys = result.keys
      return if 'error_response'.in?(keys)

      key = "#{request_params[:method].gsub(/\./, '_')}_response"
      return unless key.in?(keys) && keys.size == 1

      result[key][:err_code].to_i.zero?
    end

  end
end